# An "Audit Event Source" dialog
#
# Copyright (C) 2008 Red Hat, Inc.  All rights reserved.
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.  You should have
# received a copy of the GNU General Public License along with this program; if
# not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
# Floor, Boston, MA 02110-1301, USA.  Any Red Hat trademarks that are
# incorporated in the source code or documentation are not subject to the GNU
# General Public License and may only be used or replicated with the express
# permission of Red Hat, Inc.
#
# Red Hat Author: Miloslav Trmac <mitr@redhat.com>
from gettext import gettext as _

import gtk
import gobject

from dialog_base import DialogBase
import event_source
import util

__all__ = ('SourceDialog')

class SourceDialog(DialogBase):

    '''An event source dialog.'''

    __gsignals__ = { 'destroy': (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
                                 ()),
                     'apply': (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, ()) }

    _glade_widget_names = ('source_apply',
                           'source_log', 'source_log_label',
                           'source_path', 'source_path_browse',
                           'source_path_label',
                           'source_type_file', 'source_type_log')

    def __init__(self, parent):
        DialogBase.__init__(self, 'source_dialog', parent)
        self.__running_modally = False

        self.__log_store = gtk.ListStore(gobject.TYPE_STRING)
        self.source_log.set_model(self.__log_store)
        cell = gtk.CellRendererText()
        self.source_log.pack_start(cell, True)
        self.source_log.set_attributes(cell, text = 0)

        util.connect_and_run(self.source_type_log, 'toggled',
                             self.__source_type_log_toggled)
        util.connect_and_run(self.source_type_file, 'toggled',
                             self.__source_type_file_toggled)
        self._setup_browse_button(self.source_path_browse, self.source_path,
                                  _('Audit Log File'),
                                  gtk.FILE_CHOOSER_ACTION_OPEN)
        self.window.connect('destroy', self.__window_destroy)
        self.window.connect('response', self.__window_response)

    def run(self):
        '''Run the dialog modally until it is "correctly" closed.

        Assumes the caller is handling the 'apply' signal.

        '''
        self.__running_modally = True
        self.source_apply.hide() # What does "apply" mean in a modal dialog?
        res = self.window.run()
        # If there is an error, self.__window_response() has already reported
        # it, so call self._validate_get_failure() instead of
        # self._validate_values() here.
        while (res == gtk.RESPONSE_OK and
               self._validate_get_failure() is not None):
            res = self.window.run()
        # If the response is gtk.RESPONSE_OK and the values validate,
        # self.__window_response() has already emitted 'apply'.
        self.source_apply.show()
        self.__running_modally = False
        return res

    __type_map = (('source_type_log', event_source.ClientEventSource),
                  ('source_type_file', event_source.FileEventSource))

    def load(self, main_window):
        '''Modify dialog controls to reflect main_window.'''
        client = main_window.client
        self.source_type_log.set_sensitive(client is not None)
        self.__log_store.clear()
        if client is not None:
            for filename in client.list_files():
                self.__log_store.append((filename,))
            self.source_log.set_active_iter(self.__log_store.get_iter_first())

        source = main_window.event_source
        if isinstance(source, event_source.ClientEventSource):
            source_type = event_source.ClientEventSource
            util.set_combo_option(self.source_log, source.filename)
        else:
            assert isinstance(source, event_source.FileEventSource), \
                'Unexpected event source'
            source_type = event_source.FileEventSource
            self.source_path.set_text(source.path)
        self._radio_set(source_type, self.__type_map)

    def save(self, main_window):
        '''Modify main_window to reflect dialog state.'''
        if self.source_type_log.get_active():
            assert main_window.client is not None
            it = self.source_log.get_active_iter()
            assert it is not None
            filename = self.__log_store.get_value(it, 0)
            source = event_source.ClientEventSource(main_window.client,
                                                    filename)
        else:
            path = self.source_path.get_text()
            source = event_source.FileEventSource(path)
        main_window.event_source = source

    def _validate_get_failure(self):
        if self.source_type_file.get_active():
            path = self.source_path.get_text()
            try:
                f = open(path)
                f.close()
            except IOError, e:
                return (_('Error opening %s: %s') % (path, e.strerror), None,
                        self.source_path)
        return None

    def __source_type_log_toggled(self, *_):
        util.set_sensitive_all(self.source_type_log.get_active(),
                               self.source_log_label, self.source_log)

    def __source_type_file_toggled(self, *_):
        util.set_sensitive_all(self.source_type_file.get_active(),
                               self.source_path_label, self.source_path,
                               self.source_path_browse)

    def __window_destroy(self, *_):
        self.emit('destroy')
        return False

    def __window_response(self, unused_widget, response):
        if response in (gtk.RESPONSE_APPLY, gtk.RESPONSE_OK):
            if not self._validate_values():
                return # Don't destroy the dialog
            self.emit('apply')
        if not self.__running_modally and response != gtk.RESPONSE_APPLY:
            self.destroy()

gobject.type_register(SourceDialog)
